'DVI Graphics Shield Demo #1
'Copyright (C) 2012 Rayslogic.com, LLC
'Shows graphics and text in 640x480 resolution
'Demonstrates basic shape drawing in 24-bit color and also a virtual TV tile driver
'Also demonstrates drawing 24, 8, and 4 bit-per pixel bitmaps from SD card
'  SD card is only required to show bitmaps, you can partially run the demo without it
'Note:  If not using Propeller Platform, you may need to change pin settings in SSD1963 driver and also SD pins below


CON 'Crystal settings
  _clkmode = xtal1+PLL16X
  _xinfreq = 5_000_000

OBJ
  lcd: "4p3_Plugin_driver1k"   'NOTE:  Be sure to change pin settings in this driver to match your setup if not using Propeller Platform!
  sd: "fsrw"     'SD card driver

CON 'SD card pin settings
  SD_Do=0
  SD_Clk=1
  SD_Di=2
  SD_Cs=3

CON 'constants for virtual text screen
  x_tiles = lcd#xPixels/16   
  y_tiles = lcd#yPixels/16
  n_tiles = x_tiles*y_tiles 

  cols=x_tiles
  rows=y_tiles
  screensize = cols * rows
  lastrow = screensize - cols

VAR  'variables for the virtual text screen
  word  screen[x_tiles * y_tiles]
  long  colors[64]
  long  row,col,flag,color
  
VAR 'buffer for bitmap line of pixels
  long lineBuffer[864*3/4]' buffer for line of an image  'needs to be long aligned
  long BmpPalette[256]

PUB Main 

  'start DVI driver
  lcd.start(@screen, @colors) 
  lcd.SetBacklightPWM(128) 'turn backlight on 100%

  'run demo
  SSD1963_Demo

PUB SSD1963_Demo|i, j, k     
  'Prepare TV mode colors
  setcolors(@palette)      

  'demo loop
  repeat
  
    'Fill screen fast way
    lcd.FillScreen(lcd.RGB(1,64,128))
    'lcd.DrawRectangle(lcd#rgb_white,0,0,639,479)     'draws white boarder around whole screen 
    
    'Set random pixels
    repeat 10000
      lcd.SetPixel(lcd.RGB(?k,?k,?k),||?k//lcd#xPixels,||?k//lcd#yPixels)
     
    'Draw random lines
    repeat 500  
      lcd.DrawLine(lcd.RGB(?k,?k,?k),||?k//lcd#xPixels,||?k//lcd#yPixels,||?k//lcd#xPixels,||?k//lcd#yPixels)
      
    'Draw random circles
    repeat 100
      lcd.DrawSolidCircle(lcd.RGB(?k,?k,?k),40+||?k//(lcd#xPixels-60),40+||?k//(lcd#yPixels-80),||?k//40)
     
    'Draw random triangles
    repeat 50
      lcd.DrawSolidTriangle(lcd.RGB(?k,?k,?k),||?k//lcd#xPixels,||?k//lcd#yPixels,||?k//lcd#xPixels,||?k//lcd#yPixels,||?k//lcd#xPixels,||?k//lcd#yPixels)
     
    'Draw random rectangles
    repeat 50
      lcd.DrawSolidRectangle(lcd.RGB(?k,?k,?k),||?k//lcd#xPixels,||?k//lcd#yPixels,||?k//lcd#xPixels,||?k//lcd#yPixels)

    'Draw random ROM font characters
    repeat 2000
      lcd.DrawRomFontChar(?k//8,||?k//(lcd#xPixels-16),||?k//(lcd#yPixels-16),?k//256)

    'Mounting SD card now
    Print($100)  'clear text screen 
    PrintStr(string("Mounting SD Card...")) 'put text on screen buffer
    Lcd.UpdateTextScreen  'copy text screen buffer to screen
    repeat 10  'try a few times to mount the SD card
      i:=sd.mount_explicit(SD_DO, SD_CLK, SD_DI,SD_CS)
      if i==0
         quit
      waitcnt(cnt+clkfreq/4)
    
    'Showing some 24-bit bitmap images from SD card   
    k:=string("4test1.bmp")
    repeat j from "1" to "4"
      byte[k+5]:=j   'alter bitmap filename to pick different bitmaps
      Print($100)  'clear text screen 
      PrintStr(string("Showing Bitmap: ")) 'put text on screen buffer
      PrintStr(k)
      Lcd.UpdateTextScreen  'copy text screen buffer to screen
      waitcnt(cnt+clkfreq*2) 'wait 2 seconds
      BMP(0,0,k)   'show the bitmap
      repeat 500 'set pixel where user touches screen for a while
        i:=lcd.getTouch
        if i>0
          lcd.SetPixel(lcd#rgb_white,i>>16,i&$FFF)
          waitcnt(cnt+clkfreq/1000)
      'waitcnt(cnt+clkfreq*2)


  


PUB BMP(bmpX,bmpY,sFilename)|i,j,r,g,b,c,d,xm,ym
'Draw a bitmap file onto the screen
    'return
    'lcd.SetWindow(0,lcd#xpixels-1,0,lcd#ypixels-1)
    
    'note that bitmaps are stored from bottom line to top line!
    i:=\sd.popen(sFilename,"r") 'open image for reading
    if i<>0
      return i   
    sd.pread(@bfType,2) 'read bmp header
    sd.pread(@bfSize,4) 'read bmp header  
    sd.pread(@bfReserved1,4) 'read bmp header  
    sd.pread(@bfOffBits,16) 'read bmp header
    sd.pread(@biPlanes,4)
    sd.pread(@biCompression,24)


    if (biHeight<1) or (biWidth<1)
      return -1  'some kind of problem
      
    case biBitCount
      24:  'true color (3 bytes per pixel)         
        repeat j from biHeight+bmpY-1 to bmpY 
          'process image row by row
          lcd.SetWindow(bmpX,constant(lcd#xpixels-1),j,constant(lcd#ypixels-1))
          lcd.write_CMD($2C) 'Begin memory write
          sd.pRead(@lineBuffer,biWidth*3)    
          lcd.WriteBitmapLine(@lineBuffer,biWidth,24,0)
          i:=(biWidth*3)//4
          repeat i  'note that each line is padded to have an even # of longs  
            sd.pgetc
 
      8:  '256-Palette mode
        'first need to read in palette  (rgbquad (4 bytes) * # pallete entries)
        'calculate actual bytes in palette
        i:=bfOffBits-54
        'read in palette
        sd.pread(@BmpPalette,i)

        repeat j from biHeight+bmpY-1 to bmpY  
          lcd.SetWindow(bmpX,constant(lcd#xpixels-1),j,constant(lcd#ypixels-1))
          lcd.write_CMD($2C) 'Begin memory write
          sd.pRead(@lineBuffer,biWidth)
          lcd.WriteBitmapLine(@lineBuffer,biWidth,8,@BmpPalette) 
          i:=biWidth//4
          repeat i  'note that each line is padded to have an even # of longs  
            sd.pgetc
      
      4:  '16-Palette mode
        'first need to read in palette  (rgbquad (4 bytes) * # pallete entries)
        'calculate actual bytes in palette
        i:=bfOffBits-54
        'read in palette
        sd.pread(@BmpPalette,i)
        'now paint picture
        repeat j from biHeight+bmpY-1 to bmpY  
          lcd.SetWindow(bmpX,constant(lcd#xpixels-1),j,constant(lcd#ypixels-1))
          lcd.write_CMD($2C) 'Begin memory write
          sd.pRead(@lineBuffer,biWidth>>1)
          lcd.WriteBitmapLine(@lineBuffer,biWidth,4,@BmpPalette) 
          i:=(biWidth>>1)//4
          repeat i  'note that each line is padded to have an even # of longs  
            sd.pgetc

    sd.pclose
    return 0

PUB setcolors(colorptr) | i, fore, back

'' Override default color palette
'' colorptr must point to a list of up to 8 colors
'' arranged as follows:
''
''               fore   back
''               ------------
'' palette  byte color, color     'color 0
''          byte color, color     'color 1
''          byte color, color     'color 2
''          ...

  repeat i from 0 to 7
    fore := byte[colorptr][i << 1]
    back := byte[colorptr][i << 1 + 1]
    colors[i << 1]     := fore << 24 + back << 16 + fore << 8 + back
    colors[i << 1 + 1] := fore << 24 + fore << 16 + back << 8 + back

    
PRI PrintStr(stringptr)

'' Print a zero-terminated string

  repeat strsize(stringptr)
    print(byte[stringptr++])

PUB print(c) | i, k

  case c
    $00..$FF:           'character?
      k := color << 1 + c & 1
      i := k << 10 + $200 + c & $FE
      screen[row * cols + col] := i
      screen[(row + 1) * cols + col] := i | 1
      if ++col == cols
        newline

    $100:               'clear screen?
      wordfill(@screen, $220, screensize)
      col := row := 0

    $108:               'backspace?
      if col
        col--

    $10D:               'return?
      newline

    $110..$11F:         'select color?
      color := c & $F


' New line

PRI newline : i

  col := 0
  if (row += 2) == rows
    row -= 2
    'scroll lines
    repeat i from 0 to rows-3
'      wordmove(@screen[i*cols], @screen[(i+2)*cols], cols)
    'clear new line
'    wordfill(@screen[(rows-2)*cols], $200, cols<<1)


' Data

PRI dec(value) | i

'' Print a decimal number

  if value < 0
    -value
    print("-")

  i := 1_000_000_000

  repeat 10
    if value => i
      print(value / i + "0")
      value //= i
      result~~
    elseif result or i == 1
      print("0")
    i /= 10

PRI hex(value, digits)

'' Print a hexadecimal number

  value <<= (8 - digits) << 2
  repeat digits
    print(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))
    

DAT    'palette for text colors

                        '       fore   back
                        '       color  color
palette                 byte    $07,   $0A    '0    white / dark blue
                        byte    $07,   $BB    '1    white / red
                        byte    $9E,   $9B    '2   yellow / brown
                        byte    $04,   $07    '3     grey / white
                        byte    $3D,   $3B    '4     cyan / dark cyan
                        byte    $6B,   $6E    '5    green / gray-green
                        byte    $BB,   $CE    '6      red / pink
                        byte    $3C,   $0A    '7     cyan / blue

DAT     'Data for reading bitmap files

BMPHeader  byte 'Mostly using info from here:  http://www.fortunecity.com/skyscraper/windows/364/bmpffrmt.html
bfType byte "B","M"  ' 19778
bfSize long 0
bfReserved1 word 0
bfReserved2 word 0
bfOffBits long 54
biSize long 40
biWidth long 0
biHeight long 10
biPlanes word 1
biBitCount word 24
biCompression long 0
biSizeImage long 0
biXPelsPerMeter long 0
biYPelsPerMeter long 0
biClrUsed long 0
biClrImportant long 0
                        