'PCA9554 I2C 8-Bit I/O Expansion Driver
'Copyright 2007 Raymond Allen   
'This is a stripped down version of Michael Green's "Basic_i2c_driver" found on "Object Exchange"
'It only reads and writes bytes
'It is coded to use pins 28 and 29, which are used by the Prop at boot time to access the EEPROM
'
'WARNING:  You should set all pins to HIGH (i.e., call OUT(255)) before using pins as inputs.  Otherwise, an input signal to a pin may be connected to ground and fry the chip!
'           (but, note that all pins HIGH is the power-on state)

'This is all in SPIN, so it doesn't use up a cog.


CON
   ACK      = 0                        ' I2C Acknowledge
   NAK      = 1                        ' I2C No Acknowledge
   Xmit     = 0                        ' I2C Direction Transmit
   Recv     = 1                        ' I2C Direction Receive
   SCL      = 28             'I2C clock pin#
   SDA      = 29             'I2C data pin#

   AD5337_Address = $0C       'DAC
   AD7999_Address = $29       'ADC
   TouchAdd = $48  'Address of touchscreen controller
   MMA7455_Address = $1D

PUB GetAccelX
return Get2ByteAccelReg($00)

PUB GetAccelY
return Get2ByteAccelReg($02)

PUB GetAccelZ
return Get2ByteAccelReg($04)

PUB GetAccelX8
return Get1ByteAccelReg($06)

PUB GetAccelY8
return Get1ByteAccelReg($07)

PUB GetAccelZ8
return Get1ByteAccelReg($08) 

PRI Get2ByteAccelReg(nReg)|m,v 
  m:=GetAccelReg(nReg)
  'need to swap bytes
  v:=(m>>8)+(m&$3)<<8
  return (v<<22)~>22  'result is signed 10 bit...

PRI Get1ByteAccelReg(nReg)|m,v    
  m:=GetAccelReg(nReg)
  'need to swap bytes
  v:=m>>8
  return (v<<24)~>24  'result is signed 8 bit...


PUB GetAccelReg(reg) |ackbit ,x
  'Retrieve contents of acclerometer register
   START
   ackbit:=Write(MMA7455_Address<<1)
   ackbit:=Write(reg) 
   STOP
   x:=in(MMA7455_Address)
   'Going to swap high and low bytes...
   return x       

PUB WriteAccelReg(reg,data) |ackbit 
   START
   ackbit:=Write(MMA7455_Address<<1)
   ackbit := (ackbit << 1) | ackbit:=Write(reg)
   ackbit := (ackbit << 1) | ackbit:=Write(data)
   return (ackbit==ACK)
   STOP
   
PUB GetADC(channel):AnalogIn|i
  'channel:  0=extra ADC Input, 1=x-axis of accelerometer, 2=y-axis, 3=z-axis 
  ConfigADC(1<<(channel+4))  'set input channel
  return ReadADC  'read and return channel value                                

PRI ConfigADC(data)|ackbit
   START
   ackbit:=Write(AD7999_Address<<1)
   ackbit := (ackbit << 1) | ackbit:=Write(data) 
   return (ackbit==ACK)
   STOP  

PRI ReadADC:AnalogIn|i
    i:=in(AD7999_Address)
    i<<=20  'strip away channel info
    i>>=24  'make to 256 bit
    return i


PUB StartTouch     |ackbit
   START
   ackbit:=Write(TouchAdd<<1)
   if ackbit<>ACK
     return false  
   ackbit:=Write(0) 
   return (ackbit==ACK)
   STOP  

PUB GetTouchX:x     |ackbit
   START
   ackbit:=Write(TouchAdd<<1)
   ackbit:=Write($C0) '$CE=%%11001110  'measure x, 8-bit, irq disabled
                      '$C0  =%%11000000  'measure x, 12-bit, irq enabled
   STOP
   x:=in(TouchAdd) 
   return x

PUB GetTouchY:x     |ackbit
   START
   ackbit:=Write(TouchAdd<<1)
   ackbit:=Write($D0) '$DE=%%11011110  'measure y, 8-bit, irq disabled
                      '$D0  =%%11010000  'measure y, 12-bit, irq enabled
   STOP
   x:=in(TouchAdd) 
   return x

PUB GetTouchZ1:z1|ackbit
   START
   ackbit:=Write(TouchAdd<<1)
   ackbit:=Write($E0) '$EE=%%11011110  'measure z1, 8-bit, irq disabled
                      '$E0  =%%11010000  'measure z1, 12-bit, irq enabled
   STOP
   z1:=in(TouchAdd)
   return z1
PUB  GetTouchZ2:z2|ackbit  
   START
   ackbit:=Write(TouchAdd<<1)
   ackbit:=Write($F0) '$FE=%%11011110  'measure z1, 8-bit, irq disabled
                      '$F0  =%%11010000  'measure z1, 12-bit, irq enabled
   STOP
   z2:=in(TouchAdd)
   return z2
    


PUB SetDAC(channel,level):ackbit  'output given level via DAC output #1
   'channel 0=backlight control, channel 1= extra DAC output
   'write to output port register
   START
   ackbit:=Write(AD5337_address<<1)
   ackbit := (ackbit << 1) | ackbit:=Write(1<<channel)  'channel 1
   ackbit := (ackbit << 1) | ackbit:=Write($20+level>>4)  '
   ackbit := (ackbit << 1) | ackbit:=Write(level<<4)  '
   STOP
   return ackbit'(ackbit==ACK)  

PUB IN(address):data|ackbit
    START
    ackbit:=Write((address<<1)+1)
    if (ackbit==ACK)  
      data:=Read(ACK)
    else
      data:=-1   'return negative to indicate read failure
    STOP
    return data
    

PUB OUT(address, command, data):ackbit       'Commands:  1=output port register, 2=polarity inversion register, 3=config register
   'negative data not sent
   'write to output port register
   START
   ackbit:=Write(address<<1)
   ackbit:=Write(command)  'output port register command=1
   if (data=>0)
      ackbit := (ackbit << 1) | Write(data)
   STOP
   return (ackbit==ACK)

            

PUB Start'(SCL) | SDA                   ' SDA goes HIGH to LOW with SCL HIGH
   outa[SCL]~~                         ' Initially drive SCL HIGH
   dira[SCL]~~
   outa[SDA]~~                         ' Initially drive SDA HIGH
   dira[SDA]~~
   outa[SDA]~                          ' Now drive SDA LOW
   outa[SCL]~                          ' Leave SCL LOW
  
PUB Stop'(SCL) | SDA                    ' SDA goes LOW to HIGH with SCL High
   outa[SCL]~~                         ' Drive SCL HIGH
   outa[SDA]~~                         '  then SDA HIGH
   dira[SCL]~                          ' Now let them float
   dira[SDA]~                          ' If pullups present, they'll stay HIGH

PUB Write(data) : ackbit '(SCL, data) : ackbit | SDA
'' Write i2c data.  Data byte is output MSB first, SDA data line is valid
'' only while the SCL line is HIGH.  Data is always 8 bits (+ ACK/NAK).
'' SDA is assumed LOW and SCL and SDA are both left in the LOW state.
   ackbit := 0 
   data <<= 24
   repeat 8                            ' Output data to SDA
      outa[SDA] := (data <-= 1) & 1
      outa[SCL]~~                      ' Toggle SCL from LOW to HIGH to LOW
      outa[SCL]~
   dira[SDA]~                          ' Set SDA to input for ACK/NAK
   outa[SCL]~~
   ackbit := ina[SDA]                  ' Sample SDA when SCL is HIGH
   outa[SCL]~
   outa[SDA]~                          ' Leave SDA driven LOW
   dira[SDA]~~

PUB Read(ackbit):data'(SCL, ackbit): data | SDA
'' Read in i2c data, Data byte is output MSB first, SDA data line is
'' valid only while the SCL line is HIGH.  SCL and SDA left in LOW state.
   data := 0
   dira[SDA]~                          ' Make SDA an input
   repeat 8                            ' Receive data from SDA
      outa[SCL]~~                      ' Sample SDA when SCL is HIGH
      data := (data << 1) | ina[SDA]
      outa[SCL]~
   outa[SDA] := ackbit                 ' Output ACK/NAK to SDA
   dira[SDA]~~
   outa[SCL]~~                         ' Toggle SCL from LOW to HIGH to LOW
   outa[SCL]~
   dira[SDA]~
   repeat 8                            ' Receive data from SDA
      outa[SCL]~~                      ' Sample SDA when SCL is HIGH
      data := (data << 1) | ina[SDA]
      outa[SCL]~
   outa[SDA] := NAK'ackbit                 ' Output ACK/NAK to SDA
   
   dira[SDA]~~
   outa[SCL]~~                         ' Toggle SCL from LOW to HIGH to LOW
   outa[SCL]~
   outa[SDA]~                          ' Leave SDA driven LOW


  'I don't think we need this as Prop must have already initialized the I2C bus on Pins 28,29
PUB Initialize'(SCL) | SDA              ' An I2C device may be left in an
   outa[SCL] := 1                       '   reinitialized.  Drive SCL high.
   dira[SCL] := 1
   dira[SDA] := 0                       ' Set SDA as input
   repeat 9
      outa[SCL] := 0                    ' Put out up to 9 clock pulses
      outa[SCL] := 1
      if ina[SDA]                      ' Repeat if SDA not driven high
         quit                          '  by the EEPROM

 